@@ -13,7 +13,7 @@ df <- data.frame(
   weirdness = rnorm(n, mean = 4, sd = 2),
   genre = sample(c("Rock", "Jazz", "Country"), size = n, replace = T),
   reaction = sample(c("Love", "Huh", "Dislike", "Mixed"),
-                    size = n, replace = T),
+                    size = n, replace = T, prob = c(4, 4, 4, 6)),
   guitar_flag = sample(c(0, 1), size = n, replace = T),
   drum_flag = sample(c(0, 1, NA), size = n, replace = T,
                      prob = c(0.45, 0.45, 0.1)),
@@ -176,8 +176,9 @@ test_that("date columns are found and converted", {
 
 test_that("date columns are found, converted, and dummified", {
   d_clean <- prep_data(d = d_train, outcome = is_ween, song_id, convert_dates = "categories")
+  d_clean_dummied_months <- d_clean[grepl("date_col_month", names(d_clean))]
   # Should find 11 month names in names of prepped data as all but one get dummied
-  expect_equal(sum(purrr::map_lgl(month.abb, ~ any(grepl(.x, names(d_clean))))), 11)
+  expect_equal(sum(purrr::map_lgl(month.abb, ~ any(grepl(.x, names(d_clean_dummied_months))))), 11)
   expect_true("date_col_dow_Thu" %in% names(d_clean))
   expect_equal(sort(unique(d_clean$col_DTS_month_Sep)), c(0, 1))
   expect_true(all(2006:2008 %in% d_clean$col_DTS_year))
@@ -240,6 +241,11 @@ test_that("impute works with partial/extra params", {
                                        numeric_params = list(knn_K = 5),
                                        the_best_params = "Moi!")),
     regexp = "the_best_params")
+
+  d_clean <- prep_data(d = d_train, outcome = is_ween, song_id,
+                       impute = list(numeric_method = "locfimpute"))
+  m <- missingness(d_clean)
+  expect_true(all(m$percent_missing == 0))
 })
 
 test_that("rare factors go to other by default", {
@@ -250,7 +256,7 @@ test_that("rare factors go to other by default", {
                length(exp) - 1)
   d_clean <- prep_data(d = d_train, outcome = is_ween, song_id, make_dummies = FALSE)
   expect_true(all(exp %in% levels(d_clean$state)))
-  exp <- c("Dislike", "Huh", "Love", "Mixed", "missing")
+  exp <- c("Huh", "Love", "Mixed", "missing", "other")
   expect_true(all(exp %in% levels(d_clean$reaction)))
 })
 
@@ -289,10 +295,10 @@ test_that("dummy columns are created as expected", {
   d_clean <- prep_data(d = d_train, outcome = is_ween, song_id,
                        convert_dates = "none", make_dummies = TRUE,
                        add_levels = FALSE, collapse_rare_factors = FALSE)
-  exp <- c("genre_Jazz", "genre_Rock", "genre_missing")
+  exp <- c("genre_Country", "genre_Jazz", "genre_Rock")
   n <- names(dplyr::select(d_clean, dplyr::starts_with("genre")))
   expect_true(all(exp %in% n))
-  exp <- c("reaction_Huh", "reaction_Love", "reaction_Mixed",
+  exp <- c("reaction_Dislike", "reaction_Huh", "reaction_Love",
            "reaction_missing" )
   n <- names(dplyr::select(d_clean, dplyr::starts_with("reaction")))
   expect_true(all(n == exp))
@@ -321,7 +327,7 @@ test_that("recipe attr is a recipe class object", {
 })
 
 test_that("Training data are stored entirely in the recipe", {
-  expect_equivalent(attr(d_prep, "recipe")$template, d_train)
+  expect_equivalent(attr(d_prep, "recipe")$orig_data, d_train)
 })
 
 test_that("warning is given when ignored columns have missingness", {
@@ -353,7 +359,10 @@ test_that("prep_data applies recipe from training on test data", {
   expect_equal(d_reprep, d_reprep2)
   expect_equal(unique(d_reprep$weirdness[is.na(d_test$weirdness)]),
                mean(d_train$weirdness, na.rm = TRUE))
-  expect_true(all(d_reprep$genre_missing[is.na(d_test$genre)] == 1))
+  # When missing is the reference level
+  expect_true(all(d_reprep$genre_Country[is.na(d_test$genre)] == 0 &&
+                    d_reprep$genre_Rock[is.na(d_test$genre)] == 0 &&
+                    d_reprep$genre_Jazz[is.na(d_test$genre)] == 0))
 })
 
 test_that("Unignored variables present in training but not deployment error if needed", {
@@ -603,6 +612,10 @@ test_that("prep_data warns for all unique character columns and adds the to igno
 })
 
 test_that("prep_data doesn't check for all-unique columns in predict", {
+  d_train <-
+    d_train %>%
+    dplyr::mutate_if(is.factor, as.character)
+
   # Additional test of this in the last code chunk of healthcareai.Rmd
   d_prep <- prep_data(d = d_train, outcome = is_ween, song_id,
                       convert_dates = "none", make_dummies = FALSE)
@@ -625,6 +638,30 @@ test_that("prep_data gets rid of logicals, when no outcome", {
   expect_false(any(purrr::map_lgl(prepped_test, is.logical)))
 })
 
+test_that("test ref levels input - and step_dummy_hcai attributes", {
+  d_clean <- prep_data(d = d_train, outcome = is_ween, song_id,
+                       make_dummies = list(genre = "Jazz", reaction = "Dislike"))
+  expect_false("genre_Jazz" %in% names(d_clean))
+  expect_false("reaction_Dislike" %in% names(d_clean))
+
+  # Ref levels and dummies are stored in step_dummy_hcai
+  steps <- (d_clean %>% attr("recipe"))$step
+  loc <- purrr::map_lgl(steps, ~{
+    class(.x) %>% first() == "step_dummy_hcai"
+  }
+  )
+  dummies <- steps[loc][[1]]$dummies
+  ref_levels <- steps[loc][[1]]$ref_levels
+
+  expect_true("genre_Country" %in% dummies$dummy)
+  expect_true("reaction_Love" %in% dummies$dummy)
+  expect_true("Jazz" %in% dummies$ref)
+  expect_true("Dislike" %in% dummies$ref)
+
+  expect_equal(ref_levels, c(genre = "Jazz", reaction = "Dislike",
+                             state = "CA"))
+})
+
 test_that("no_prep dominates", {
   noprep <- prep_data(animals_train, outcome = y, no_prep = TRUE)
   expect_true(attr(attr(noprep, "recipe"), "no_prep"))
@@ -647,3 +684,10 @@ test_that("prep_data doesn't throw warning if DTS character column is unique", {
   )
   expect_true(sum(admitted_dTS_cols) > 1)
 })
+
+test_that("make_dummies throws error if not logical or list", {
+  expect_error(
+    prep_data(d_train, make_dummies = "Test"),
+    "step_dummies must be logical or list"
+  )
+})
